from panda3d.core import *
from direct.showbase.ShowBase import ShowBase
from direct.gui.OnscreenText import OnscreenText

import sys, os, math, gzip, time, ConfigParser
#import struct, itertools, array, ctypes
#import array as pyarray
#import itertools

loadPrcFileData("","sync-video #f")
class Application(ShowBase):
    def __init__(self):
        ShowBase.__init__(self)
        self.accept("escape", sys.exit)        
        base.setFrameRateMeter(True)
        render.setAntialias(AntialiasAttrib.MAuto)
        
        self.init_lights()        
        self.init_camera()
        self.init_collision()
        self.init_room()
        self.init_player()        

        self.accept("w", self.switch, [0,0,1,0])
        self.accept("a", self.switch, [1,0,0,0])
        self.accept("s", self.switch, [0,0,0,1])
        self.accept("d", self.switch, [0,1,0,0])

    def init_room(self):
        self.blob, self.lgray, self.dgray, self.flashcount, self.colors = get_colors()
        self.flashing = False; self.flashcolor = -1; self.flashing_nodes = []
        base.win.setClearColor(Vec4(self.lgray[0],self.lgray[1],self.lgray[2],1)) # Background color

        self.textObject = OnscreenText("1.lvl, 1.lvl, 1.lvl, 1.lvl, 1.lvl", pos = (-0.5, 0.85), scale = 0.07, fg = (1,1,1,1))
        self.path = "rooms/default"
        self.path = self.path.replace("/",os.sep)
        self.reader = objectReader(scale=1.0,colors=self.colors)
        self.room = currentRoom()
        StartTime = time.time()
        self.reader.read(os.path.join(self.path,"1.lvl"),self.room,"level",collision=True)
        self.color = self.reader.color
        self.reader.build(collisions=False)

        self.roomnode = render.attachNewNode(self.room.geomnode)
        self.room.collider = self.roomnode.attachNewNode(self.room.colnode)

        EndTime = (time.time() - StartTime)/60
        print "Geometry done in %s seconds." %(EndTime)
        print "Room %s has %s geometries." %(self.room.num, self.roomnode.node().getNumGeoms())
        self.textObject.setText("%s.lvl, %s, %s, %s, %s" %(self.room.num, self.room.right, self.room.left, self.room.up, self.room.down))
        #self.roomnode.showTightBounds()

    def init_collision(self):
        """ create the collision system """
        base.cTrav = CollisionTraverser()
        base.pusher = CollisionHandlerPusher()

    def init_player(self):
        self.player = playertype(self.room, self.roomnode, self)
        self.reader.createBox(1.0, [0.0, 0.0, 0.0], self.room.color, self.player, collision=False)        
        self.reader.build(collisions=False)
        self.playernode = render.attachNewNode(self.player.geomnode)
        self.player.colnode.addSolid(CollisionSphere(0,0,0,0.25))
        self.player.collider = self.playernode.attachNewNode(self.player.colnode)
        base.cTrav.addCollider(self.player.collider,base.pusher)
        base.pusher.addCollider(self.player.collider,self.playernode)
        self.player.collider.show()
        self.playernode.setPos(0.0, -8.0, 0.0)
        self.player.playernode = self.playernode
        self.player.startPlayer()        

    def init_lights(self):
        pntLight = PointLight("point")
        pntLight.setColor(Vec4(1.0, 1.0, 1.0, 1.0))
        #pntLight.setAttenuation((1, 0, 0.01))
        pntNode = render.attachNewNode(pntLight)
        pntNode.setPos(0, 2, 3)
        pntNode.reparentTo(self.cam)
        render.setLight(pntNode)

        ambLight = AmbientLight("ambient")
        ambLight.setColor(Vec4(0.5, 0.5, 0.5, 0.5))
        ambNode = render.attachNewNode(ambLight)
        render.setLight(ambNode)

    def init_camera(self):
        self.cam.setZ(30)
        self.cam.setHpr(0,-90,0)        
        self.cam.node().getLens().setFov(75)

    def switch(self,right,left,up,down):        
        if right:
            new = self.room.right
        elif left:
            new = self.room.left
        elif up:
            new = self.room.up
        else: # down
            new = self.room.down
        sides = []
        if new == "-1.lvl" or new == "0.lvl":
            return
        StartTime = time.time()
        self.reader.read(os.path.join(self.path,new),self.room,"level",collision=True)
        self.color = self.reader.color
        self.reader.build(collisions=False)
        EndTime = (time.time() - StartTime)/60
        print "Geometry done in %s seconds." %(EndTime)
        self.textObject.setText("%s.lvl, %s, %s, %s, %s" %(self.room.num, self.room.right, self.room.left, self.room.up, self.room.down))
        if self.room.blob:
            c = self.blob
        else:
            if self.room.num < 45:
                c = self.lgray
            else:
                c = self.dgray
        base.win.setClearColor(Vec4(c[0],c[1],c[2],1))
        self.playernode.setColor(self.color[0], self.color[1], self.color[2], 1.0)
        self.roomnode.setColor(self.color[0], self.color[1], self.color[2], 1.0)
        if self.room.flashing:            
            self.flashing_nodes.append(self.playernode)
            self.flashing_nodes.append(self.roomnode)
            self.flashing = True
            self.check_flashing(start=True)
        else:
            if self.flashing is True:
                self.flashing_nodes.remove(self.roomnode)                
                self.flashing_nodes.remove(self.playernode)
                if self.flashing_nodes == []:
                    self.flashing = False
                    self.check_flashing(stop=True)
                self.roomnode.setColor(self.color[0], self.color[1], self.color[2], 1.0)
        print "Room %s has %s geometries." %(self.room.num, self.roomnode.node().getNumGeoms())

    def check_flashing(self, start=False, stop=False):
        if start:
            self.taskMgr.doMethodLater(0.05, self.flashingTask, 'world::flashingTask')
        if stop:
            self.taskMgr.remove('world::flashingTask')

    def flashingTask(self, task):        
        self.flashcolor += 1
        if self.flashcolor >= self.flashcount:
            self.flashcolor = 0        
        for flashing_node in self.flashing_nodes: # Set all the flashing objects.
            c = self.colors[self.flashcolor]
            flashing_node.setColor(c[0], c[1], c[2], 1.0)
        if self.flashing is False: # Stop looping if no flashing objects are onscreen.
            return task.done            
        return task.again

def get_colors(): # This needs to be replaced with a color-reading function from the .ini file.
    colors = [[231,231,49,1],[181,214,66,1],[132,222,23,1],[181,255,156,1],
              [132,255,148,1],[24,189,16,1],[33,66,0,1],[0,49,8,1],[0,0,107,1],
              [90,107,255,1],[82,181,255,1],[165,82,222,1],[239,82,239,1],
              [255,82,82,1],[173,82,24,1],[132,49,8,1],[206,82,33,1],
              [255,156,41,1],[255,206,132,1],[255,231,165,1],[255,255,255,1],
              [255,247,82,1],[255,222,74,1],[206,206,206,0],[33,222,24,0],
              [255,132,255,0],[214,181,66,0],[165,181,49,0],[99,214,115,0],
              [57,57,57,0],[148,148,148,0],[181,173,255,0],[0,0,0,0]]
    invc = 1/255.0
    for c in colors:
        for i in range(3):
            c[i] = c[i] * invc
    blob = colors[17]
    lgray = colors[23]
    dgray = colors[30]
    count = 32
    return blob, lgray, dgray, count, colors

class objectReader(object):
    def __init__(self, scale=1.0, colors=[]):        
        self.polys = [[3,2,1,0],[4,5,6,7],[5,4,0,1],[6,5,1,2],[7,6,2,3],[4,7,3,0]]        
        self.norms = [[-1.0, 0.0, 0.0],[1.0, 0.0, 0.0],[0.0, 0.0, -1.0],[0.0, 1.0, 0.0],[0.0, 0.0, 1.0],[0.0, -1.0, 0.0]]
        self.uvs = [[0.0, 1.0],[0.0, 0.0],[1.0, 0.0],[1.0, 1.0]]
        self.scale = scale
        self.default_scale = scale
        self.colors = colors
        
    def makeBox(self, center, sides, boxtype, collision=False):
        self.setCurrent(boxtype) # "verts", "holes", or "fakes"
        scale = self.scale * 0.5 
        box = [center[0]-scale,center[0]+scale, center[1]-scale, center[1]+scale, center[2]-scale, center[2]+scale]
        # box format is [minX,maxX,minY,maxY,minZ,maxZ]            
        self.verts = [ [box[0],box[2],box[4]], [box[0],box[3],box[4]],
                       [box[0],box[3],box[5]], [box[0],box[2],box[5]],
                       [box[1],box[2],box[4]], [box[1],box[3],box[4]],
                       [box[1],box[3],box[5]], [box[1],box[2],box[5]] ]
        if collision:
            self.object.colnode.addSolid(CollisionBox(Point3(center[0], center[1], center[2]), scale*1.25, scale*1.25, scale*1.25))
        for side in range(6):
            if sides[side] == 0:
                continue
            
            norm = self.norms[side]
            poly = []
            for vi in self.polys[side]:
                self.object.current_verts.append([i for i in self.verts[vi]])
                poly.append(len(self.object.current_verts)-1)                
            self.object.current_vnorms.append([i for i in norm])
            self.object.current_polys.append([i for i in poly])                       
            self.object.current_uvs.append([[j for j in i] for i in self.uvs])            

    def setCurrent(self, currentType):
        if currentType == "verts" or self.type != "level":
            self.object.current_verts = self.object.verts
            self.object.current_polys = self.object.polys
            self.object.current_vnorms = self.object.vnorms            
            self.object.current_uvs = self.object.uvs
        elif currentType == "holes":
            self.object.current_verts = self.object.hole_verts
            self.object.current_polys = self.object.hole_polys
            self.object.current_vnorms = self.object.hole_vnorms            
            self.object.current_uvs = self.object.hole_uvs
        elif currentType == "fakes":
            self.object.current_verts = self.object.fake_verts
            self.object.current_polys = self.object.fake_polys
            self.object.current_vnorms = self.object.fake_vnorms            
            self.object.current_uvs = self.object.fake_uvs

    def setScale(self, scale):
        self.scale = scale

    def setColor(self, color):
        self.color = color

    def setObject(self, obj):
        self.object = obj

    def clearObjectGeom(self):
        self.object.verts = []
        self.object.polys = []        
        self.object.vnorms = []
        self.object.uvs = []

    def createBox(self, scale, center, color, obj, collision=False):
        self.setObject(obj)
        self.clearObjectGeom()
        self.setScale(scale)
        self.setColor(self.colors[color])
        self.makeBox( center, [1,1,1,1,1,1], "verts", collision=collision)

    def read(self, filepath, gameObject, objectType, scale=None, collision=False):
        if not scale:
            scale = self.scale
        else:
            self.scale = scale
        self.object = gameObject
        self.type = objectType
        self.clearObjectGeom()
        f = open(filepath)
        lines = f.readlines()        
        f.close()
        lines = [l.rstrip("\r\n") for l in lines if len(l)]
        if self.type == "level":
            self.object.hole_verts = []
            self.object.hole_polys = []            
            self.object.hole_vnorms = []
            self.object.hole_uvs = []
            self.object.fake_verts = []
            self.object.fake_polys = []            
            self.object.fake_vnorms = []
            self.object.fake_uvs = []
            
            self.object.num = int(lines[0])
            self.object.right = lines[1]
            self.object.up = lines[2]
            self.object.left = lines[3]
            self.object.down = lines[4]
            self.object.castle = int(lines[5])
            self.object.cup = lines[6]
            self.object.entry = int(lines[7])
            self.object.edn = lines[8]
            self.object.color = int(lines[9])-1
            self.object.blob = int(lines[10])
            self.object.rwall = int(lines[11])
            self.object.lwall = int(lines[12])
            self.object.narrow = int(lines[13])
            geomstart = 14
            geomend = len(lines)
            width = len(lines[14])
            depth = geomend - geomstart
            self.object.width = width
            self.object.depth = depth
            self.object.data_offset = geomstart
            self.object.map = [lines[di] for di in range(geomstart, geomend)]
            if self.object.color >= len(self.colors): # Flashing
                self.color = self.colors[0]
                self.object.flashing = True
            else:
                self.color = self.colors[self.object.color]
                self.object.flashing = False

            if collision:
                if self.object.colnode.getNumSolids:
                    self.object.colnode.clearSolids()
        elif self.type == "object":
            w,d,scale,ox,oz = lines[0].split(",")
            w = float(w)
            d = float(d)
            scale = float(scale)
            ox = float(ox)
            oz = float(oz)
            width = int(w)
            depth = int(d)
            self.object.bounds.ox = ox*scale
            self.object.bounds.oy = oy*scale
            self.object.bounds.mx = ((w*0.5))*scale
            self.object.bounds.nx = (-(w*0.5))*scale
            self.object.bounds.nz = (-(d*0.5))*scale
            self.object.bounds.mz = ((d*0.5))*scale
            geomstart = 1
            geomend = len(lines)
        #["right", "left", "bottom", "back", "top", "front"]
        empty = [" ", "0", "H"]
        filled = ["B", "X", "O", "1"]
        for di in range(geomstart,geomend):            
            for wi in range(width):                
                box = lines[di][wi]
                #if self.type == "level":
                #    print di, wi, len(self.object.map), repr(box), depth, geomend
                #    self.object.map[di-geomstart][wi] = box
                if box in empty:                    
                    continue                
                center = ((wi-(width*0.5)+0.5)*scale, (1.5+depth-di)*scale, 0) # center it
                sides = [0,0,1,0,1,0]
                boxtype = "verts"
                if (di == geomstart) or ((lines[di-1][wi] in empty) and di > geomstart):
                    sides[3] = 1 # back = +Y
                if (di == geomend-1) or ((lines[di+1][wi] in empty) and di < geomend-1):
                    sides[5] = 1 # front = -Y
                if (wi == 0) or ((lines[di][wi-1] in empty) and wi > 0):
                    sides[0] = 1 # right = -X
                if (wi == width-1) or ((lines[di][wi+1] in empty) and wi < width-1):
                    sides[1] = 1 # left = +X
                """
                if box == "X":
                    boxtype = "fakes"
                elif box == "O":
                    boxtype = "holes"
                """
                if box == "X":
                    #boxtype = "fakes"
                    self.makeBox(center, sides, boxtype, collision=False)
                else:
                    self.makeBox(center, sides, boxtype, collision=collision)
        #for line in self.object.map:            
            #print line    

    def clear(self):
        self.verts = []
        self.object = None
        self.type = ""
        self.color = []
        self.scale = self.default_scale

    #import array as pyarray
    #import itertools
    def buildTEST(self, order="per_poly"): # Not working, but keep to test in dev build.
        """Create the geometry from the submitted arrays"""
        verts = self.object.verts
        polys = self.object.polys
        if self.object.geomnode.getNumGeoms():
            self.object.geomnode.removeAllGeoms()

        format=GeomVertexFormat.getV3n3cpt2() # Proper order of columns: vert, norm, color, texcoord   
        vdata=GeomVertexData('geom', format, Geom.UHStatic)

        vertex = GeomVertexWriter(vdata, 'vertex')
        normal = GeomVertexWriter(vdata, 'normal')
        color = GeomVertexWriter(vdata, 'color')
        texcoord = GeomVertexWriter(vdata, 'texcoord')
        tri=GeomTriangles(Geom.UHStatic)
        
        merged = [] # Combine lists.  Not sure which order is correct.  Both tests assume one comprehensive list, rather than per-poly chunks.
        if order == "per_column":
            for p in polys:
                for vi in p:
                    merged.append(verts[vi])
            for norm in self.object.vnorms:
                for i in range(4):
                    merged.append(norm)
            for p in polys:
                for vi in p:
                    merged.append([self.color[0], self.color[1], self.color[2], 1.0])
            for uvset in self.object.uvs:
                for uv in uvset:
                    merged.append(uv)
                    
        elif order == "per_poly":
            for poly_index in range(len(polys)):
                p = polys[poly_index]
                for vi in p:
                    merged.append(verts[vi])
                for i in range(4):
                    merged.append(self.object.vnorms[poly_index])
                for vi in p:
                    merged.append([self.color[0], self.color[1], self.color[2], 1.0])
                for uv in self.object.uvs[poly_index]:
                    merged.append(uv)
                    
        merged_array = pyarray.array('f',list(itertools.chain.from_iterable(merged))) # Flatten merged list and put in array format      
        vdata.modifyArray(0).modifyHandle().setData(merged_array)#.tostring())
        
        #vdata.modifyArray(0).modifyHandle().copyDataFrom(merged_array)#.tostring()) # Per rdb, this will work in dev build.
            
        for poly_index in range(len(polys)): # Process the tris
            p = polys[poly_index]
            tri.addVertices(p[0],p[1],p[3])
            tri.addVertices(p[1],p[2],p[3])
        
            if poly_index == len(polys)-1:
                tri.closePrimitive()            
                geom=Geom(vdata)
                geom.addPrimitive(tri)
                self.object.geomnode.addGeom(geom)
        self.clear()
        
    def build(self, collisions=False): # Best so far.  Keep.
        """Create the geometry from the submitted arrays"""
        verts = self.object.verts
        polys = self.object.polys
        if self.object.geomnode.getNumGeoms():
            self.object.geomnode.removeAllGeoms()
        if collisions:
            if self.object.colnode.getNumSolids:
                self.object.colnode.clearSolids()

        format=GeomVertexFormat.getV3n3cpt2()        
        vdata=GeomVertexData('geom', format, Geom.UHStatic)

        vertex = GeomVertexWriter(vdata, 'vertex')
        normal = GeomVertexWriter(vdata, 'normal')
        color = GeomVertexWriter(vdata, 'color')
        texcoord = GeomVertexWriter(vdata, 'texcoord')
        tri=GeomTriangles(Geom.UHStatic)        
           
        for poly_index in range(len(polys)):
            p = polys[poly_index]
            uvs = self.object.uvs[poly_index]       
            norm = self.object.vnorms[poly_index]
            
            v1 = verts[p[0]]
            v2 = verts[p[1]]
            v3 = verts[p[2]]
            v4 = verts[p[3]]
            vertex.addData3f(v1[0], v1[1], v1[2])
            vertex.addData3f(v2[0], v2[1], v2[2])
            vertex.addData3f(v3[0], v3[1], v3[2])
            vertex.addData3f(v4[0], v4[1], v4[2])
            
            normal.addData3f(norm[0], norm[1], norm[2])
            normal.addData3f(norm[0], norm[1], norm[2])
            normal.addData3f(norm[0], norm[1], norm[2])
            normal.addData3f(norm[0], norm[1], norm[2])
            
            color.addData4f(self.color[0], self.color[1], self.color[2], 1.0)
            color.addData4f(self.color[0], self.color[1], self.color[2], 1.0)
            color.addData4f(self.color[0], self.color[1], self.color[2], 1.0)
            color.addData4f(self.color[0], self.color[1], self.color[2], 1.0)

            texcoord.addData2f(uvs[0][0], uvs[0][1])
            texcoord.addData2f(uvs[1][0], uvs[1][1])
            texcoord.addData2f(uvs[2][0], uvs[2][1])
            texcoord.addData2f(uvs[3][0], uvs[3][1]) 

            tri.addVertices(p[0],p[1],p[3])
            tri.addVertices(p[1],p[2],p[3])
            if collisions: # Option to set up per-poly collisions
                cPoly = CollisionPolygon(Point3(v1[0], v1[1], v1[2]), Point3(v2[0], v2[1], v2[2]), Point3(v4[0], v4[1], v4[2]))
                self.object.colnode.addSolid(cPoly)
                cPoly = CollisionPolygon(Point3(v2[0], v2[1], v2[2]), Point3(v3[0], v3[1], v3[2]), Point3(v4[0], v4[1], v4[2]))
                self.object.colnode.addSolid(cPoly)

            if poly_index == len(polys)-1:
                tri.closePrimitive()            
                geom=Geom(vdata)
                geom.addPrimitive(tri)
                self.object.geomnode.addGeom(geom)
        self.clear()

"""
# http://code.google.com/p/python-panda3d-examples/
Functions to load and read configuration from an ini file.
"""
_config = {} # The configuration dictionary

# Load the configuration from the given file
def load_config(file):
    config_parser = ConfigParser.ConfigParser() # Create a ConfigParser
    config_parser.read(file) # Read the ini file
    # Store in the '_config' dictionary the options
    for section in config_parser.sections():
        _config[section] = {}
        for option in config_parser.options(section):
            _config[section][option] = config_parser.get(section, option)

# These functions returns an option value given its section, for example 'fire-particles',
# and its name, for example 'pool_size'. 
def get_string(section, option):
    return str(_config[section][option])

def get_int(section, option):
    return int(_config[section][option])

def get_float(section, option):
    return float(_config[section][option])


def _init_colors(paths): # Create the textures
    flashcount = 0
    paths.file = paths.dirname + "/" + paths.settings
    paths_file = read_ini(paths.file)
    c_blob = ini_readInteger(paths_file, "colors", "blob", 18)
    c_lgray = ini_readInteger(paths_file, "colors", "lgray", 24)
    c_dgray = ini_readInteger(paths_file, "colors", "dgray", 31)
    numcolors = ini_readInteger(paths_file, "colors", "count", 33)
    c_black = numcolors
    c_flash = numcolors + 1
    for i in range(1,numcolors):
        #if image exist(i) = 1 then delete image i        
        a = ini_readString(paths_file, "colors", "c%s"%(i), "255,255,255,1")
        r,g,b,f = a.split(",")
        r = int(r)
        g = int(g)
        b = int(b)
        if colormode == 1: # Black and white - doesn't accurately reflect b&w colors on 2600
            r = (r + g + b)/3
            g = r
            b = r
        elif colormode == 2: # X-ray - handling for blob may need work
            if i == c_lgray or i == c_dgray:
                r = 255
                g = 255
                b = 255
            else:
                r = 0
                g = 0
                b = 0
        elif colormode == 3: # Muted palette
            r = (r/3)*2
            g = (g/3)*2
            b = (b/3)*2     
        flashcount += int(f) #!
        dot( i-1,0,rgb(r,g,b)) #!
    for i in range(1,numcolors):
        get_image( i,i-1,0,i,1,0) #!

class currentRoom(object):
    def __init__(self):
        # Variables for the current room
        self.id = -1     # level object id
        self.num = -1   # current room number as int, for comparisons
        self.up = ""      # filename for up room .lvl file
        self.down = ""    # same for down
        self.left = ""    # same for left
        self.right = ""   # same for right
        self.color = -1  # room color
        self.castle = -1 # if room contains a castle, this is the castle number (corresponds to key number)
        self.rwall = -1  # whether the room has a right barrier
        self.lwall = -1  # whether the room has a left barrier
        self.blob = -1   # whether the room is a 'dark' torch-lit room
        self.narrow = -1 # whether room has passages of same width or depth as player object
        self.entry = -1 # If room is a castle entry, this stores the castle number
        self.tempcolor = -1 # Store room color to restore after game-end flashing
        self.cup = ""
        self.edn = ""
        self.geomnode = GeomNode('room::geometry')
        self.colnode = CollisionNode('room::collision')
        self.flashing = False

class pathtype(object):
    def __init__(self):
        # File paths used in the game
        self.shapes = ""
        self.setup = ""
        self.settings = ""        
        self.rooms = ""
        self.enemies = ""
        self.kingdoms = ""
        self.props = ""        
        self.temp = "" # Internal use for room changes
        self.file = "" # Internal use for save/load
        self.xray = ""
        self.dirname = ""

class vectype(object):
    def __init__(self):
        self.x = 0.0
        self.y = 0.0
        self.z = 0.0

class arraycoords(object):
    def __init__(self):
        # Used by random room placement, to quickly find an open location in a room
        self.ax = -1
        self.az = -1

class bboxtype(object):
    def __init__(self):
        self.nx = 0.0 # min x
        self.mx = 0.0 # max x
        self.nz = 0.0 # min z
        self.mz = 0.0 # max z
        self.ox = 0.0 # offset x (used for magnet attraction, dragon-bite locations)
        self.oz = 0.0 # offset z

class carrytype(object):
    def __init__(self):
        self.x = 0.0
        self.z = 0.0
        self.id = -1
        # Stores id and offset delta of carried object

class playertype(object):
    def __init__(self, room, roomnode, gameApp):
        self.id = -1
        self.state = -1 # 0=dead/deactivated; 1=active; 2=bat-carried playerdead
        self.color = -1 # Color (usually duplicates room color, but can be different)
        self.num = -1 # Room number       
        self.pos = vectype() # Position
        self.carry = carrytype() # Delta and id for carry object
        self.bridge = -1 # bridge activation switch
        self.swallow = -1 # Which dragon has eaten a dead player
        self.geomnode = GeomNode('player::geometry')
        self.colnode = CollisionNode('player::collision')
        self.playernode = None

        self.forward = False
        self.backward = False
        self.left = False
        self.right = False
        self.room = room
        self.roomnode = roomnode
        self.gameApp = gameApp        
        bounds = self.roomnode.getTightBounds()
        self.upper_bound = bounds[1][1]
        self.lower_bound = bounds[0][1]
        self.right_bound = bounds[1][0]
        self.left_bound = bounds[0][0]
        
    def moveTask(self,task):        
        dt = task.time - self.time
        if self.forward == True:
            self.playernode.setY(self.playernode, self.playernode.getY(self.playernode)
                             + self.movSens * self.fast * dt)
        if self.backward == True:
            self.playernode.setY(self.playernode, self.playernode.getY(self.playernode)
                             - self.movSens * self.fast * dt)
        if self.left == True:
            self.playernode.setX(self.playernode, self.playernode.getX(self.playernode)
                             - self.movSens * self.fast * dt)
        if self.right == True:
            self.playernode.setX(self.playernode, self.playernode.getX(self.playernode)
                             + self.movSens * self.fast * dt)
        x = self.playernode.getX()
        y = self.playernode.getY()
        l = self.left_bound
        r = self.right_bound
        u = self.upper_bound
        w = self.lower_bound
        """
        colliding = self.getGridSquare()
        if colliding:
            new = Vec2(self.prev_x, self.prev_y) - Vec2(x, y)
            new.normalize()
            #self.playernode.setPos((x+new[0])*dt,(y+new[1])*dt,0.0)
            #self.playernode.setPos(new[0],new[1],0.0)
            self.playernode.setX(x + (new[0]) * 1.5)# * 0.5)
            self.playernode.setY(y + (new[1]) * 1.5)# * 0.5)
            x = self.playernode.getX()
            y = self.playernode.getY()
            print x,y
            #self.playernode.setX(self.prev_x)
            #self.playernode.setY(self.prev_y)
            return task.cont
        """
        
        if x < l:
            self.gameApp.switch(0,1,0,0)
            self.playernode.setX(r)           
        elif x > r:
            self.gameApp.switch(1,0,0,0)
            self.playernode.setX(l)            
        elif y > u:
            self.gameApp.switch(0,0,1,0)
            self.playernode.setY(w)            
        elif y < w:
            self.gameApp.switch(0,0,0,1)
            self.playernode.setY(u)       
        self.time = task.time
        #self.prev_x = self.playernode.getX()
        #self.prev_y = self.playernode.getY()
        return task.cont

    def getGridSquare(self, scale=1.0):
        # center = ((wi-(width*0.5)+0.5)*scale, (1.5+depth-di)*scale, 0)
        x = self.playernode.getX()
        y = self.playernode.getY()
        empty = [" ", "F"]
        blocked = ["B", "H"]
        width = self.room.width
        depth = self.room.depth
        oneOverScale = 1.0/scale
        data_offset = self.room.data_offset
        grid = self.room.map
        #x = (wi-(width*0.5)+0.5)*scale
        #y = (1.5+depth-di)*scale
        wi = ((x*oneOverScale)-0.5) + (width*0.5)
        #di = depth - ((y*oneOverScale)-1.5)        
        di = abs((y*oneOverScale)-(depth*0.5))
        wi = int(wi)
        di = int(di)
        if wi > width-1 or wi < 0 or di > depth-1 or di < 0:
            return 0
        #wi = max(min(int(wi),width-1),0)
        #di = max(min(int(di),depth-1),0)        
        return grid[di][wi] in blocked
        #print wi, di, grid[di][wi]#width, depth, x, y

    def startPlayer(self):
        self.movSens = 10.0
        self.fast = 1.0
        self.time = 0
        self.prev_x = self.playernode.getX()
        self.prev_y = self.playernode.getY()
        self.gameApp.accept("arrow_up", setattr, [self, "forward", True])        
        self.gameApp.accept("arrow_up-up", setattr, [self, "forward", False])
        self.gameApp.accept("arrow_down", setattr, [self, "backward", True])        
        self.gameApp.accept("arrow_down-up", setattr, [self, "backward", False])
        self.gameApp.accept("arrow_right", setattr, [self, "right", True])        
        self.gameApp.accept("arrow_right-up", setattr, [self, "right", False])
        self.gameApp.accept("arrow_left", setattr, [self, "left", True])        
        self.gameApp.accept("arrow_left-up", setattr, [self, "left", False])
        self.gameApp.taskMgr.doMethodLater(0.33333, self.moveTask, 'player::moveTask')        

    def stopPlayer(self):
        self.forward = False
        self.backward = False
        self.left = False
        self.right = False
        self.gameApp.ignore("arrow_up")
        self.gameApp.ignore("arrow_up-up")
        self.gameApp.ignore("arrow_down")
        self.gameApp.ignore("arrow_down-up")
        self.gameApp.ignore("arrow_right")
        self.gameApp.ignore("arrow_right-up")
        self.gameApp.ignore("arrow_left")
        self.gameApp.ignore("arrow_left-up")
        self.gameApp.taskMgr.remove('player::moveTask')    

class enemyroom(object):
    def __init__(self):
        self.num = -1
        self.up = ""
        self.down = ""
        self.left = ""
        self.right = ""
        self.castle = -1 # Castle number if room is castle
        self.cup = "" # Castle entry .lvl
        self.entry = -1 # Castle number if room is entry
        self.edn = "" # Castle exit .lvl

class xraytype(object):
    def __init__(self):
        self.d1 = -1
        self.d2 = -1
        self.d3 = -1
        self.x1 = -1
        self.x2 = -1
        self.x3 = -1

class dragontype(object):
    def __init__(self):
        self.id = -1
        self.state = -1 # 0=dead/deactivated; 1=active; 2=barking; 3=bat-carried & active; 4=fed; 5=bat+dead; 6=bat+bark; 7=bat+fed
        self.rm = enemyroom() # Enemy room data for room changes
        self.pos = vectype() # Position 
        self.dirx = 0.0 # Angle of motion        
        self.guard = -1 # Object dragon is guarding
        self.color = -1
        self.name = "" # Name used by ini read/write
        self.speed = 0.0
        self.bbox = bboxtype() # Bounding box (used in bat pick-up)
        self.bark = -1 # Timer variable for dragon attack state
        self.prefs = "" # Guarding preferences
        self.bite = bboxtype() # Bite zone bounds box
        self.dspeed = 0.0 # Default speed; Davetron game difficulty will alter speed variable; store it here for restoration 
        self.bounce = -1 # State variable for enemy bounce with horz movement
        self.geomnode = GeomNode('geometry')

class battype(object):
    def __init__(self):
        self.id = -1
        self.state = -1 # 0=inactive; 1=active; 2=player-carried; 3=whistle-frozen
        self.rm = enemyroom()
        self.pos = vectype()        
        self.dirx = 0.0        
        self.want = -1 # Object bat has decided to pick up
        self.carry = carrytype()
        self.color = -1
        self.name = ""
        self.speed = 0.0
        self.bbox = bboxtype()
        self.dspeed = 0.0 # Default speed; Davetron game difficulty will alter speed variable; store it here for restoration 
        self.bounce = -1 # State variable for enemy bounce with horz movement
        self.geomnode = GeomNode('geometry')

class proptype(object):
    def __init__(self):
        self.id = -1
        self.num = -1
        self.pos = vectype()
        self.state = -1 # 0=inactive; 1=active
        self.color = -1
        self.name = ""
        self.gametype = -1 # 1=default; 2=Davetron; 3=Custom/user-defined

class objtype(object):
    def __init__(self):
        self.id = -1
        self.num = -1
        self.pos = vectype()
        self.state = -1 # 0=deactivated; 1=active; 2=player-carried; 3=bat-carried
        self.key = -1 # Key number will match castle cnum; If game has a 'Master' key, it will have a value greater than the number of castles in game
        self.color = -1
        self.name = ""
        self.mag = -1 # Magnet hierarchy value or -1, if not affected by magnet
        self.bbox = bboxtype()
        self.rules = "" # Randomization rules, temporarily stored between game.ini read and final placement
        self.geomnode = GeomNode('geometry')

class castletype(object):
    def __init__(self):
        self.id = -1 # object id for associated gate
        self.crm = -1 # castle room number
        self.cnum = -1 # castle number (associated key has same number)
        self.entry = -1 # entry room
        self.state = -1 # 1=locked,0=open

def _init_paths(): # Define the file paths used in the game
    paths = pathtype()   
    paths.shapes = "objects/shape/"
    paths.setup = "objects/setup/"      
    paths.settings = paths.setup + "default/setup.ini"    
    paths.rooms = "rooms/default/"
    paths.enemies = paths.rooms + "enemies/"
    paths.kingdoms = paths.setup + "default/kingdoms.ini"
    paths.props = paths.setup + "default/props.ini"  
    paths.xray = paths.setup + "xray.ini"  
    paths.temp = ""
    paths.dirname = "" #get dir$()
    return paths
    
if __name__ == "__main__":
  gameApp = Application()
  gameApp.run()      
